@@ -1,39 +1,57 @@
-//---------------------------------------------------------------------
-//  Algorithmic Conjurings @ http://www.coyotegulch.com
-//  Evocosm -- An Object-Oriented Framework for Evolutionary Algorithms
-//
-//  evoreal.h
-//---------------------------------------------------------------------
-//
-//  Copyright 1996, 1999, 2002, 2003, 2004, 2005 Scott Robert Ladd
-//
-//  This program is free software; you can redistribute it and/or modify
-//  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
-//  (at your option) any later version.
-//  
-//  This program is distributed in the hope that it will be useful,
-//  but WITHOUT ANY WARRANTY; without even the implied warranty of
-//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//  GNU General Public License for more details.
-//  
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the
-//      Free Software Foundation, Inc.
-//      59 Temple Place - Suite 330
-//      Boston, MA 02111-1307, USA.
-//
-//-----------------------------------------------------------------------
-//
-//  For more information on this software package, please visit
-//  Scott's web site, Coyote Gulch Productions, at:
-//
-//      http://www.coyotegulch.com
-//  
-//-----------------------------------------------------------------------
+/*
+    Evocosm is a C++ framework for implementing evolutionary algorithms.
 
-#if !defined(LIBEVOCOSM_GAFLOAT_H)
-#define LIBEVOCOSM_GAFLOAT_H
+    Copyright 2011 Scott Robert Ladd. All rights reserved.
+
+    Evocosm is user-supported open source software. Its continued development is dependent
+    on financial support from the community. You can provide funding by visiting the Evocosm
+    website at:
+
+        http://www.coyotegulch.com
+
+    You may license Evocosm in one of two fashions:
+
+    1) Simplified BSD License (FreeBSD License)
+
+    Redistribution and use in source and binary forms, with or without modification, are
+    permitted provided that the following conditions are met:
+
+    1.  Redistributions of source code must retain the above copyright notice, this list of
+        conditions and the following disclaimer.
+
+    2.  Redistributions in binary form must reproduce the above copyright notice, this list
+        of conditions and the following disclaimer in the documentation and/or other materials
+        provided with the distribution.
+
+    THIS SOFTWARE IS PROVIDED BY SCOTT ROBERT LADD ``AS IS'' AND ANY EXPRESS OR IMPLIED
+    WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
+    FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SCOTT ROBERT LADD OR
+    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
+    ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+    The views and conclusions contained in the software and documentation are those of the
+    authors and should not be interpreted as representing official policies, either expressed
+    or implied, of Scott Robert Ladd.
+
+    2) Closed-Source Proprietary License
+
+    If your project is a closed-source or proprietary project, the Simplified BSD License may
+    not be appropriate or desirable. In such cases, contact the Evocosm copyright holder to
+    arrange your purchase of an appropriate license.
+
+    The author can be contacted at:
+
+          scott.ladd@coyotegulch.com
+          scott.ladd@gmail.com
+          http:www.coyotegulch.com
+*/
+
+#if !defined(LIBEVOCOSM_EVOREAL_H)
+#define LIBEVOCOSM_EVOREAL_H
 
 // libevocosm
 #include "evocommon.h"
@@ -52,7 +70,7 @@ namespace libevocosm
         data it is modifying; however, natural chromosomes do encode some
         structure in their sequence; for example, crossover appears to take place
         in specific positions along the chromosome. And while mutation doesn't
-        care about a chromosome's structure, it does affect that structure. 
+        care about a chromosome's structure, it does affect that structure.
         In context of a computer program, the structure of a chromosome isn't
         so important as the ability to logically modify its bits through
         crossover and mutation.
@@ -64,7 +82,7 @@ namespace libevocosm
         I'm aware of the VAX and other systems; this code is explicitly
         non-portable outside implementations of IEC 60559/IEEE-754.
     */
-    class evoreal : protected globals 
+    class evoreal : protected globals
     {
     public:
         //! Creation constructor
