@@ -1,45 +1,60 @@
-//---------------------------------------------------------------------
-//  Algorithmic Conjurings @ http://www.coyotegulch.com
-//  Roulette Wheel -- A Tool for Weighted Selection
-//
-//  roulette.cpp
-//---------------------------------------------------------------------
-//
-//  Copyright 1996, 1999, 2002, 2003, 2004, 2005 Scott Robert Ladd
-//
-//  This program is free software; you can redistribute it and/or modify
-//  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
-//  (at your option) any later version.
-//  
-//  This program is distributed in the hope that it will be useful,
-//  but WITHOUT ANY WARRANTY; without even the implied warranty of
-//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//  GNU General Public License for more details.
-//  
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the
-//      Free Software Foundation, Inc.
-//      59 Temple Place - Suite 330
-//      Boston, MA 02111-1307, USA.
-//
-//-----------------------------------------------------------------------
-//
-//  For more information on this software package, please visit
-//  Scott's web site, Coyote Gulch Productions, at:
-//
-//      http://www.coyotegulch.com
-//  
-//-----------------------------------------------------------------------
+/*
+    Evocosm is a C++ framework for implementing evolutionary algorithms.
+
+    Copyright 2011 Scott Robert Ladd. All rights reserved.
+
+    Evocosm is user-supported open source software. Its continued development is dependent
+    on financial support from the community. You can provide funding by visiting the Evocosm
+    website at:
+
+        http://www.coyotegulch.com
+
+    You may license Evocosm in one of two fashions:
+
+    1) Simplified BSD License (FreeBSD License)
+
+    Redistribution and use in source and binary forms, with or without modification, are
+    permitted provided that the following conditions are met:
+
+    1.  Redistributions of source code must retain the above copyright notice, this list of
+        conditions and the following disclaimer.
+
+    2.  Redistributions in binary form must reproduce the above copyright notice, this list
+        of conditions and the following disclaimer in the documentation and/or other materials
+        provided with the distribution.
+
+    THIS SOFTWARE IS PROVIDED BY SCOTT ROBERT LADD ``AS IS'' AND ANY EXPRESS OR IMPLIED
+    WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
+    FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SCOTT ROBERT LADD OR
+    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
+    ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+    The views and conclusions contained in the software and documentation are those of the
+    authors and should not be interpreted as representing official policies, either expressed
+    or implied, of Scott Robert Ladd.
+
+    2) Closed-Source Proprietary License
+
+    If your project is a closed-source or proprietary project, the Simplified BSD License may
+    not be appropriate or desirable. In such cases, contact the Evocosm copyright holder to
+    arrange your purchase of an appropriate license.
+
+    The author can be contacted at:
+
+          scott.ladd@coyotegulch.com
+          scott.ladd@gmail.com
+          http:www.coyotegulch.com
+*/
 
-// Standard C++ Library
 #include <stdexcept>
-
-// libcoyotl
-#include "libcoyotl/validator.h"
-using namespace libcoyotl;
+#include <cstring>
 
 // libevocosm
+#include "validator.h"
 #include "roulette.h"
 using namespace libevocosm;
 
@@ -59,13 +74,13 @@ roulette_wheel::roulette_wheel(const vector<double> & a_weights, double a_min_we
     for (size_t i = 0; i < m_size; ++i)
     {
         m_weights[i]   = fabs(a_weights[i]);
-        
+
         if (m_weights[i] < m_min_weight)
             m_weights[i] = m_min_weight;
         else
             if (m_weights[i] > m_max_weight)
                 m_weights[i] = m_max_weight;
-        
+
         m_total_weight += m_weights[i];
     }
 
@@ -83,19 +98,19 @@ roulette_wheel::roulette_wheel(const double * a_weights, size_t a_size, double a
     validate_not(m_size,size_t(0),"Roulette wheel can not have zero size");
     validate_less(m_min_weight,m_max_weight,"Minimum weight must be less than maximum");
     validate_greater(m_min_weight,0.0,"Minimum weight must be > 0");
-    
+
     m_weights = new double[m_size];
 
     for (size_t i = 0; i < m_size; ++i)
     {
         m_weights[i]   = fabs(a_weights[i]);
-        
+
         if (m_weights[i] < m_min_weight)
             m_weights[i] = m_min_weight;
         else
             if (m_weights[i] > m_max_weight)
                 m_weights[i] = m_max_weight;
-        
+
         m_total_weight += m_weights[i];
     }
 
@@ -126,7 +141,7 @@ roulette_wheel & roulette_wheel::operator = (const roulette_wheel & a_source)
         m_min_weight   = a_source.m_min_weight;
         m_max_weight   = a_source.m_max_weight;
     }
-    
+
     return *this;
 }
 
@@ -140,7 +155,7 @@ roulette_wheel::~roulette_wheel()
 double roulette_wheel::set_weight(size_t a_index, double a_weight)
 {
     validate_less(a_index,m_size,"invalid roulette wheel index");
-    
+
     a_weight = fabs(a_weight);
 
     if (a_weight < m_min_weight)
@@ -148,10 +163,10 @@ double roulette_wheel::set_weight(size_t a_index, double a_weight)
     else
         if (a_weight > m_max_weight)
             a_weight = m_max_weight;
-    
+
     m_total_weight -= m_weights[a_index];
     m_total_weight += a_weight;
-    
+
     double res = m_weights[a_index];
     m_weights[a_index] = a_weight;
 
@@ -168,16 +183,16 @@ double roulette_wheel::get_weight(size_t a_index) const
 // retrieve a random index
 size_t roulette_wheel::get_index() const
 {
-    double choice = g_random.get_rand_real2() * m_total_weight;
+    double choice = g_random.get_real() * m_total_weight;
 
     size_t i = 0;
-    
+
     while ((i < m_size) && (choice > m_weights[i]))
     {
         choice -= m_weights[i];
         ++i;
     }
-    
+
     return i;
 }
 
