@@ -1,49 +1,61 @@
-//---------------------------------------------------------------------
-//  Algorithmic Conjurings @ http://www.coyotegulch.com
-//  Evocosm -- An Object-Oriented Framework for Evolutionary Algorithms
-//
-//  scaler.h
-//---------------------------------------------------------------------
-//
-//  Copyright 1996, 1999, 2002, 2003, 2004, 2005 Scott Robert Ladd
-//
-//  This program is free software; you can redistribute it and/or modify
-//  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
-//  (at your option) any later version.
-//  
-//  This program is distributed in the hope that it will be useful,
-//  but WITHOUT ANY WARRANTY; without even the implied warranty of
-//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//  GNU General Public License for more details.
-//  
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the
-//      Free Software Foundation, Inc.
-//      59 Temple Place - Suite 330
-//      Boston, MA 02111-1307, USA.
-//
-//-----------------------------------------------------------------------
-//
-//  For more information on this software package, please visit
-//  Scott's web site, Coyote Gulch Productions, at:
-//
-//      http://www.coyotegulch.com
-//  
-//-----------------------------------------------------------------------
+/*
+    Evocosm is a C++ framework for implementing evolutionary algorithms.
+
+    Copyright 2011 Scott Robert Ladd. All rights reserved.
+
+    Evocosm is user-supported open source software. Its continued development is dependent
+    on financial support from the community. You can provide funding by visiting the Evocosm
+    website at:
+
+        http://www.coyotegulch.com
+
+    You may license Evocosm in one of two fashions:
+
+    1) Simplified BSD License (FreeBSD License)
+
+    Redistribution and use in source and binary forms, with or without modification, are
+    permitted provided that the following conditions are met:
+
+    1.  Redistributions of source code must retain the above copyright notice, this list of
+        conditions and the following disclaimer.
+
+    2.  Redistributions in binary form must reproduce the above copyright notice, this list
+        of conditions and the following disclaimer in the documentation and/or other materials
+        provided with the distribution.
+
+    THIS SOFTWARE IS PROVIDED BY SCOTT ROBERT LADD ``AS IS'' AND ANY EXPRESS OR IMPLIED
+    WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
+    FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SCOTT ROBERT LADD OR
+    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
+    ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+    The views and conclusions contained in the software and documentation are those of the
+    authors and should not be interpreted as representing official policies, either expressed
+    or implied, of Scott Robert Ladd.
+
+    2) Closed-Source Proprietary License
+
+    If your project is a closed-source or proprietary project, the Simplified BSD License may
+    not be appropriate or desirable. In such cases, contact the Evocosm copyright holder to
+    arrange your purchase of an appropriate license.
+
+    The author can be contacted at:
+
+          scott.ladd@coyotegulch.com
+          scott.ladd@gmail.com
+          http:www.coyotegulch.com
+*/
 
 #if !defined(LIBEVOCOSM_SCALER_H)
 #define LIBEVOCOSM_SCALER_H
 
-// Standard C Library
-#include <cmath>
-
-// Standard C++
-#include <limits>
-#include <algorithm>
-
 // libevocosm
 #include "organism.h"
+#include "stats.h"
 
 namespace libevocosm
 {
@@ -81,19 +93,6 @@ namespace libevocosm
             \param a_population - A population of organisms
         */
         virtual void scale_fitness(vector<OrganismType> & a_population) = 0;
-        
-        //! Invert a population's fitness values
-        /*!
-            Scales a population's fitness values 
-        */
-        void invert(vector<OrganismType> & a_population)
-        {
-            double base = min_element(a_population.begin(), a_population.end())->fitness()
-                        + max_element(a_population.begin(), a_population.end())->fitness();
-            
-            for (typename vector<OrganismType>::iterator organism = a_population.begin(); organism != a_population.end(); ++organism)
-               organism->fitness() = base - organism->fitness();
-        }                
     };
 
     //! A do-nothing scaler
@@ -145,51 +144,33 @@ namespace libevocosm
         virtual void scale_fitness(vector<OrganismType> & a_population)
         {
             // calculate max, average, and minimum fitness for the population
-            double max_fitness = std::numeric_limits<double>::min();
-            double min_fitness = std::numeric_limits<double>::max();
-            double avg_fitness = 0.0;
-            
-            for (typename vector<OrganismType>::iterator org = a_population.begin(); org != a_population.end(); ++org)
-            {
-                // do we have a new maximum?
-                if (org->fitness() > max_fitness)
-                    max_fitness = org->fitness();
-                
-                // do we have a new minimum?
-                if (org->fitness() < min_fitness)
-                    min_fitness = org->fitness();
-                
-                // accumulate for average
-                avg_fitness += org->fitness();
-            }
-            
-            avg_fitness /= double(a_population.size());
-            
+            fitness_stats<OrganismType> stats(a_population);
+
             // calculate coefficients for fitness scaling
             double slope;
             double intercept;
             double delta;
-            
-            if (min_fitness > ((m_fitness_multiple * avg_fitness - max_fitness) / (m_fitness_multiple - 1.0)))
+
+            if (stats.getMin() > ((m_fitness_multiple * stats.getMean() - stats.getMax()) / (m_fitness_multiple - 1.0)))
             {
                 // normal scaling
-                delta = max_fitness - avg_fitness;
-                slope = (m_fitness_multiple - 1.0) * avg_fitness / delta;
-                intercept = avg_fitness * (max_fitness - m_fitness_multiple * avg_fitness) / delta;
+                delta = stats.getMax() - stats.getMean();
+                slope = (m_fitness_multiple - 1.0) * stats.getMean() / delta;
+                intercept = stats.getMean() * (stats.getMax() - m_fitness_multiple * stats.getMean()) / delta;
             }
             else
             {
                 // extreme scaling
-                delta = avg_fitness - min_fitness;
-                slope = avg_fitness / delta;
-                intercept = -min_fitness * avg_fitness / delta;
+                delta = stats.getMean() - stats.getMin();
+                slope = stats.getMean() / delta;
+                intercept = -stats.getMin() * stats.getMean() / delta;
             }
-            
+
             // adjust fitness values
-            for (typename vector<OrganismType>::iterator org = a_population.begin(); org != a_population.end(); ++org)
-                org->fitness() = slope * org->fitness() + intercept;
+            for (int n = 0; n < a_population.size(); ++n)
+                a_population[n].fitness = slope * a_population[n].fitness + intercept;
         }
-        
+
     private:
         double m_fitness_multiple;
     };
@@ -220,17 +201,14 @@ namespace libevocosm
         */
         virtual void scale_fitness(vector<OrganismType> & a_population)
         {
-            // Find minimum fitness
-            // Note that organisms sort in reverse order of fitness, such that
-            // the "maximum" value has the smallest fitness.
-            double min_fitness = min_element(a_population.begin(), a_population.end())->fitness();
-            
+            fitness_stats<OrganismType> stats(a_population);
+
             // assign new fitness values
-            for (typename vector<OrganismType>::iterator org = a_population.begin(); org != a_population.end(); ++org)
-                org->fitness() -= min_fitness;
+            for (int n = 0; n < a_population.size(); ++n)
+                a_population[n].fitness = stats.getMin();
         }
     };
-    
+
     //! An exponential fitness scaler
     /*!
         Implements an exponential fitness scaling, whereby all fitness values are modified
@@ -265,8 +243,8 @@ namespace libevocosm
         virtual void scale_fitness(vector<OrganismType> & a_population)
         {
             // assign new fitness values
-            for (typename vector<OrganismType>::iterator org = a_population.begin(); org != a_population.end(); ++org)
-                org->fitness() = pow((m_a * org->fitness() + m_b),m_power);
+            for (int n = 0; n < a_population.size(); ++n)
+                a_population[n].fitness = pow((m_a * a_population[n].fitness + m_b),m_power);
         }
 
     private:
@@ -274,7 +252,7 @@ namespace libevocosm
         double m_b;
         double m_power;
     };
-    
+
     //! A quadratic scaler
     /*!
         Uses a quadratic equation to scale the fitness of organisms.
@@ -302,13 +280,13 @@ namespace libevocosm
         virtual void scale_fitness(vector<OrganismType> & a_population)
         {
             // adjust fitness values
-            for (typename vector<OrganismType>::iterator org = a_population.begin(); org != a_population.end(); ++org)
+            for (int n = 0; n < a_population.size(); ++n)
             {
-                double f = org->fitness();
-                org->fitness() = m_a * pow(f,2.0) + m_b * f + m_c;
+                double f = a_population[n].fitness;
+                a_population[n].fitness = m_a * pow(f,2.0) + m_b * f + m_c;
             }
         }
-        
+
     private:
         double m_a;
         double m_b;
@@ -342,42 +320,27 @@ namespace libevocosm
         */
         virtual void scale_fitness(vector<OrganismType> & a_population)
         {
-            // calculate the mean
-            double mean = 0.0;
-            for (typename vector<OrganismType>::iterator org = a_population.begin(); org != a_population.end(); ++org)
-                mean += org->fitness();
-            
-            mean /= static_cast<double>(a_population.size());
-            
-            // calculate variance
-            double variance = 0.0;
-            for (typename vector<OrganismType>::iterator org = a_population.begin(); org != a_population.end(); ++org)
-            {
-                double diff = org->fitness() - mean;
-                variance += (diff * diff);
-            }
-            
-            variance /= static_cast<double>(a_population.size() - 1);
+            fitness_stats<OrganismType> stats(a_population);
 
             // calculate 2 times the std. deviation (sigma)
-            double sigma2 = 2.0 * sqrt(variance);
-            
+            double sigma2 = 2.0 * stats.getSigma();
+
             // now assign new fitness values
             if (sigma2 == 0.0)
             {
-                for (typename vector<OrganismType>::iterator org = a_population.begin(); org != a_population.end(); ++org)
-                    org->fitness() = 1.0;
+                for (int n = 0; n < a_population.size(); ++n)
+                    a_population[n].fitness = 1.0;
             }
-            else   
+            else
             {
-                for (typename vector<OrganismType>::iterator org = a_population.begin(); org != a_population.end(); ++org)
+                for (int n = 0; n < a_population.size(); ++n)
                 {
                     // change fitness
-                    org->fitness() = (1.0 + org->fitness() / mean) / sigma2;
-                    
+                    a_population[n].fitness = (1.0 + a_population[n].fitness / stats.mean) / sigma2;
+
                     // avoid tiny or zero fitness value; everyone gets to reproduce
-                    if (org->fitness() < 0.1)
-                        org->fitness() = 0.1;
+                    if (a_population[n].fitness < 0.1)
+                        a_population[n].fitness = 0.1;
                 }
             }
         }
